const expect = require('chai').expect;
const abby = require('../lib/client');

describe('client', () => {
    let abbyClient,
        synchroniser;

    synchroniser = {
        load: () => {},
        data: [
            {
                "id": 1,
                "name": "test",
                "variants": [
                    {
                        "sequenceNumber": 0,
                        "name": "original",
                        "range": {
                            start: 0,
                            end: 100
                        },
                        "original": true,
                        "code": "original"
                    }
                ],
                "tags": [
                    {}
                ],
                "active": true,
                "code": 'feature_abc'
            },
            {
                "id": 2,
                "name": "test2",
                "variants": [
                    {
                        "sequenceNumber": 0,
                        "name": "original2",
                        "range": {
                            start: 0,
                            end: 50
                        },
                        "original": true,
                        "code": "original2"
                    }
                ],
                "tags": [
                    {}
                ],
                "active": true,
                "code": 'feature_def'
            }
        ]
    }
    beforeEach(() => {
        abbyClient = abby({
            tags: 'test, production',
            apiEndpoint: 'https://abby.io/api/v1/'
        }, synchroniser);
    });

    describe('when there is no Abby cookie', () => {
        let request,
            response;
        beforeEach(() => {
            // create the request object having no cookies
            request = {
                headers: {
                    cookie: null
                },
            };
            // create the response object for later reference
            response = {
                headers: {},
                setHeader() {
                    // do something
                    response.headers.cookie = 'uhhh cookies!';
                }
            };
            // return experiments from api
            abbyClient(request, response);
        });
        it('should set all features to request', () => {
            // check req.features
            expect(request.experiments).to.deep.equal({
                "feature_abc": "original",
                "feature_def": "original2"
            });
        });
        it('should store the evaluation result in Cookies', () => {
            // check res.cookies
            expect(response.headers.cookie).to.equal('uhhh cookies!');
        });
    });

    describe('when there are already an Abby cookie', () => {
        beforeEach(() => {
            // create the request object having cookies
            request = {
                headers: {
                    cookie: '_ga=GA1.2.77836415.1509897097; _gid=GA1.2.54182306.1519757530; optimizelyBuckets=%7B%7D; optimizelySegments=%7B%7D; constructorio_t=djF8fDk0NDM0ODFiLWU2ODMtNDRiOC04YzMyLTAzM2RiYmEyMjM4NA==; crumb=E0_d9ZZjl6rPUYJu34W2tAHANRt2wOcKeNlQD5FpOhf; ConstructorioID_client_id=9443481b-e683-44b8-8c32-033dbba22384; optimizelyEndUserId=oeu1509897097017r0.49044477643947715'
                },
            };
            // create the response object for later reference
            response = {
                headers: {},
                setHeader() {
                    // do something
                    response.headers.cookie = 'uhhh cookies!';
                }
            };
            // return experiments from api
            abbyClient(request, response);
        })
        // it('should put the client to the same non-control variation as set in the cookie', () => {
        //     //
        // });
    });
});
